==> arkhitekton-0.1.0-sources.jar: universum/studios/android/arkhitekton/view/View.kt <==
/*
 * =================================================================================================
 *                             Copyright (C) 2018 Universum Studios
 * =================================================================================================
 *         Licensed under the Apache License, Version 2.0 or later (further "License" only).
 * -------------------------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy of this License
 * you may obtain at
 *
 * 		http://www.apache.org/licenses/LICENSE-2.0
 *
 * You can redistribute, modify or publish any part of the code written within this file but as it
 * is described in the License, the software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF ANY KIND.
 *
 * See the License for the specific language governing permissions and limitations under the License.
 * =================================================================================================
 */
package universum.studios.android.arkhitekton.view

import android.arch.lifecycle.Lifecycle
import android.arch.lifecycle.LifecycleOwner
import android.arch.lifecycle.LifecycleRegistry
import android.content.res.Resources
import android.support.annotation.NonNull

/**
 * Interface that specifies base layer for views. In its basic sense a view is responsible for
 * rendering a data available via [ViewModel] to a screen and also for navigating between different
 * screens.
 *
 * Each view implementation should provide access to its current [Lifecycle] via [getLifecycle] and
 * to its associated [ViewModel] via [getViewModel].
 *
 * @param VM View model of which data will be rendered by the view.
 *
 * @author Martin Albedinsky
 * @since 1.0
 */
interface View<VM : ViewModel> : LifecycleOwner {

    /**
     * Contract for [View] element.
     */
    companion object Contract {

        /**
         * Dummy instance of view element.
         */
        @NonNull internal val DUMMY: View<ViewModel> = object : View<ViewModel> {

            /**
             * Lifecycle registry of this dummy view. Its state will never by updated.
             */
            private val lifecycle = LifecycleRegistry(this)

            /*
             */
            override fun getLifecycle() = lifecycle

            /*
             */
            override fun getViewModel() = ViewModel.DUMMY

            /*
             */
            override fun getResources(): Resources = throw UnsupportedOperationException("Cannot access Resources of DUMMY View!")
        }

        /**
         * Returns a dummy instance of [View] which may be used in cases where an instance of
         * concrete View implementation is not needed.
         *
         * The returned view provides [Lifecycle] which is always in [INITIALIZED][Lifecycle.State.INITIALIZED]
         * state and also [dummy ViewModel][ViewModel.dummy].
         *
         * **Note that the returned view instance does not provide [Resources] via [getResources]
         * method and any calls to that method will throw an exception.**
         *
         * @return Dummy view ready to be used.
         */
        @NonNull fun dummy() = DUMMY
    }

    /**
     * Returns the view model of which data this view renders to a screen.
     *
     * @return View model associated with this view.
     */
    @NonNull fun getViewModel(): ViewModel

    /**
     * Returns the resources accessible by this view instance.
     *
     * @return Application resources.
     */
    @NonNull fun getResources(): Resources
}

==> arkhitekton-0.2.0-sources.jar: universum/studios/android/arkhitekton/view/View.kt <==
/*
 * =================================================================================================
 *                             Copyright (C) 2018 Universum Studios
 * =================================================================================================
 *         Licensed under the Apache License, Version 2.0 or later (further "License" only).
 * -------------------------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy of this License
 * you may obtain at
 *
 * 		http://www.apache.org/licenses/LICENSE-2.0
 *
 * You can redistribute, modify or publish any part of the code written within this file but as it
 * is described in the License, the software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF ANY KIND.
 *
 * See the License for the specific language governing permissions and limitations under the License.
 * =================================================================================================
 */
package universum.studios.android.arkhitekton.view

import android.arch.lifecycle.Lifecycle
import android.arch.lifecycle.LifecycleOwner
import android.arch.lifecycle.LifecycleRegistry
import android.content.res.Resources
import android.support.annotation.NonNull

/**
 * Interface that specifies base layer for views. In its basic sense a view is responsible for
 * rendering a data available via [ViewModel] to a screen and also for navigating between different
 * screens.
 *
 * Each view implementation should provide access to its current [Lifecycle] via [getLifecycle] and
 * to its associated [ViewModel] via [getViewModel].
 *
 * @param VM View model of which data will be rendered by the view.
 *
 * @author Martin Albedinsky
 * @since 1.0
 */
interface View<VM : ViewModel> : LifecycleOwner {

    /*
	 * Companion ===================================================================================
	 */

    /**
     * Contract for [View] element.
     */
    companion object Contract {

        /**
         * Dummy instance which doesn't perform any logic.
         */
        @NonNull internal val DUMMY: View<*> = object : View<ViewModel> {

            /**
             * Lifecycle registry of this dummy view. Its state will never by updated.
             */
            private val lifecycle = LifecycleRegistry(this)

            /*
             */
            override fun getLifecycle() = lifecycle

            /*
             */
            override fun getViewModel() = ViewModel.DUMMY

            /*
             */
            override fun getResources(): Resources = throw UnsupportedOperationException("Cannot access Resources of DUMMY View!")
        }

        /**
         * Returns a dummy instance of [View] which may be used in cases where an instance of
         * concrete View implementation is not needed.
         *
         * The returned view provides [Lifecycle] which is always in [INITIALIZED][Lifecycle.State.INITIALIZED]
         * state and also [dummy ViewModel][ViewModel.dummy].
         *
         * **Note that the returned view instance does not provide [Resources] via [getResources]
         * method and any calls to that method will throw an exception.**
         *
         * @return Dummy view ready to be used.
         */
        @NonNull fun dummy(): View<*> = DUMMY
    }

    /*
     * Interface ===================================================================================
     */

    /*
     * Methods =====================================================================================
     */

    /**
     * Returns the view model of which data this view renders to a screen.
     *
     * @return View model associated with this view.
     */
    @NonNull fun getViewModel(): ViewModel

    /**
     * Returns the resources accessible by this view instance.
     *
     * @return Application resources.
     */
    @NonNull fun getResources(): Resources
}
